
/**********************************************************************************************/
/*Measurement System Tables.sql*/
/**********************************************************************************************/

CREATE TABLE SENSOR_TYPE
( 
	SensorTypeId         int  NOT NULL  IDENTITY ( 1,1 ) ,
	SensorTypeName       varchar(50)  NULL ,
	CONSTRAINT XPKSENSOR_TYPE PRIMARY KEY  CLUSTERED (SensorTypeId ASC)
)
go

CREATE TABLE SENSOR
( 
	SensorId             int  NOT NULL  IDENTITY ( 1,1 ) ,
	SensorName           varchar(50)  NULL ,
	SensorTypeId         int  NULL ,
	CONSTRAINT XPKSENSOR PRIMARY KEY  CLUSTERED (SensorId ASC),
	CONSTRAINT R_1 FOREIGN KEY (SensorTypeId) REFERENCES SENSOR_TYPE(SensorTypeId)
)
go

CREATE TABLE STATISTICSDATA
( 
	StatisticsDataId     int  NOT NULL  IDENTITY ( 1,1 ) ,
	SensorId             int  NULL ,
	AverageData          float  NULL ,
	MinData              float  NULL ,
	MaxData              float  NULL ,
	CONSTRAINT XPKSTATISTICSDATA PRIMARY KEY  CLUSTERED (StatisticsDataId ASC),
	CONSTRAINT R_3 FOREIGN KEY (SensorId) REFERENCES SENSOR(SensorId)
)
go

CREATE TABLE MEASUREMENTDATA
( 
	MeasurementId        int  NOT NULL  IDENTITY ( 1,1 ) ,
	MeasurementTimeStamp datetime  NULL ,
	SensorId             int  NULL ,
	MeasurementValue     float  NULL ,
	FahrenheitValue      float  NULL ,
	CONSTRAINT XPKMEASUREMENTDATA PRIMARY KEY  CLUSTERED (MeasurementId ASC),
	CONSTRAINT R_2 FOREIGN KEY (SensorId) REFERENCES SENSOR(SensorId)
)
go

	
/**********************************************************************************************/
/*GetMeasurementData.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'GetMeasurementData' 
	   AND 	  type = 'V')
	DROP VIEW GetMeasurementData
GO

CREATE VIEW GetMeasurementData
AS

SELECT
SENSOR.SensorId, 
SENSOR.SensorName, 
MEASUREMENTDATA.MeasurementId, 
MEASUREMENTDATA.MeasurementTimeStamp, 
MEASUREMENTDATA.MeasurementValue, 
MEASUREMENTDATA.FahrenheitValue

FROM MEASUREMENTDATA 
INNER JOIN SENSOR ON MEASUREMENTDATA.SensorId = SENSOR.SensorId

GO

	
/**********************************************************************************************/
/*SaveMeasurementData.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
   FROM   sysobjects 
   WHERE  name = 'SaveMeasurementData' 
   AND   type = 'P')
DROP PROCEDURE SaveMeasurementData
GO

CREATE PROCEDURE SaveMeasurementData
@SensorName varchar(50),
@MeasurementValue float
AS

DECLARE
@SensorId int

select @SensorId = SensorId from SENSOR where SensorName = @SensorName


insert into MEASUREMENTDATA (SensorId, MeasurementValue, MeasurementTimeStamp) values (@SensorId, @MeasurementValue, getdate())

GO

	
/**********************************************************************************************/
/*CalculateStatistics.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'CalculateStatistics' 
	   AND 	  type = 'TR')
	DROP TRIGGER CalculateStatistics
GO

CREATE TRIGGER CalculateStatistics ON MEASUREMENTDATA
FOR UPDATE, INSERT, DELETE 
AS

DECLARE
@SensorId int,
@AverageData float,
@MinData float,
@MaxData float

select @SensorId = SensorId from INSERTED

select @AverageData = AVG(MeasurementValue) from MEASUREMENTDATA where SensorId = @SensorId
select @MinData = MIN(MeasurementValue) from MEASUREMENTDATA where SensorId = @SensorId
select @MaxData = MAX(MeasurementValue) from MEASUREMENTDATA where SensorId = @SensorId

update STATISTICSDATA set 
AverageData = @AverageData,
MinData = @MinData,
MaxData = @MaxData   
where SensorId = @SensorId

GO

	
/**********************************************************************************************/
/*ConvertFahrenheit.sql*/
/**********************************************************************************************/
IF EXISTS (SELECT name 
	   FROM   sysobjects 
	   WHERE  name = 'ConvertFahrenheit' 
	   AND 	  type = 'TR')
	DROP TRIGGER ConvertFahrenheit
GO

CREATE TRIGGER ConvertFahrenheit ON MEASUREMENTDATA
FOR UPDATE, INSERT
AS

DECLARE
@MeasurementId int,
@MeasurementValue float,
@FahrenheitValue float

select @MeasurementId = MeasurementId from INSERTED
select @MeasurementValue = MeasurementValue from INSERTED

set @FahrenheitValue = (@MeasurementValue*9)/5 + 32;

update MEASUREMENTDATA set FahrenheitValue= @FahrenheitValue where MeasurementId= @MeasurementId

GO

	
/**********************************************************************************************/
/*Default Data.sql*/
/**********************************************************************************************/
/*
Default Data in the Database
*/

declare
@SensorTypeId int,
@SensorId int

insert into SENSOR_TYPE (SensorTypeName) values ('TC-01 Thermocouple')

select @SensorTypeId = SensorTypeId from SENSOR_TYPE where SensorTypeName='TC-01 Thermocouple'


insert into SENSOR (SensorName, SensorTypeId) values ('TC01-1', @SensorTypeId)

select @SensorId = SensorId from SENSOR where SensorName='TC01-1'


insert into STATISTICSDATA (SensorId, AverageData, MinData, maxData) values(@SensorId, 0, 0, 0)

